% macplotbug.m - a script to demonstrate Octave plotting
%                problems on a Mac.
%   There are several options for producing PDF graphics with
%   Octave 4 on a Mac. This script will demonstrate that only
%   one option does a reasonable job:
%     From Octave do
%       octave> graphics_toolkit gnuplot
%       octave> setenv GNUTERM qt
%       octave> macplotbug
%     Now the image you want is shown in the gnuplot_qt Figure 1
%     window. From this window do "Export to PDF", saving the image
%     with some appropriate name such as
%       "gnuplot_qt_Export_to_PDF.pdf".

clear all;
chdir (fileparts(mfilename("fullpath")));

%% parameters to be set
nR = 3000;      % half the (e, sig) actually generated
delta = 0.12;   % portfolio perturbation parameter
                % delta = 0.10 : close to the critical line
                % delta = 0.12 : further away
                % delta = 0.15 : further away
  % With nR = 3000 and delta = 0.12 exactly 2134 random points
  % will be plotted.

load MACPLOTBUG.mat;
%{
   Variables from MACPLOTBUG:
   A      BF     BT     P      XX     vrand
   A (253 x 4) : 2014-12-31-normalized adjusted closing prices for 4
     four large exchange traded funds:
       1. iShares 20+ Year Treasury Bond (TLT)
       2. iShares Cohen & Steers REIT (ICF)
       3. Vanguard Total Intl Bd Idx ETF (BNDX)
       4. PowerShares DB US Dollar Index Bullish (UUP)
     These prices cover the 253 market days from 2013-12-31 to
     2014-12-31.
   BF & BT : (e, sig) points along the false and true efficient
     frontiers, respectively.
   P : 2014-12-31 closing portfolios for F154 and X:
     pF154 = P(:, 1);  pX = P(:, 2);
                 TLT      ICF      BNDX     UUP
     pF154 = [ 0.13350  0.15108  0.43942  0.27600 ]';
     pX    = [ 0.11950  0.13876  0.47062  0.27112 ]';
   XX = [eXX, sigXX]';  A point on the false efficient frontier
     satisfying eXX = eF154 and sigXX < sigF154. Regardless, F154 is a
     2014-efficient unattended investment portfolio in the four exchange
     traded funds, and no investment portfolio XX actually exists 
     with the (eXX, sigXX) coordinates, even though
        eXX = E * pX  and  sigXX = sqrt(pX' * C * pX),
     whatever that means.
   vrand : state 625-vector for random number generator
%}


EF = BF(1, :); SigF = BF(2, :);
ET = BT(1, :); SigT = BT(2, :);

%% generate (e, sig) points
rand("state", vrand);
XR1 = delta * rand(4, nR);
XR2 = delta * rand(4, nR);
ZR1 = XR1 - ones(4, 1) * mean(XR1);   % deviations from the mean
ZR2 = XR2 - ones(4, 1) * mean(XR2);   % deviations from the mean
PR1 = P(:, 1) * ones(1, nR) + ZR1;    % random portfolios near P(:, 1)
PR2 = P(:, 2) * ones(1, nR) + ZR2;    % random portfolios near P(:, 2)
AR = A * [PR1, PR2];                  % 2 * nR random price vectors
RR = 100 * (AR([2 : 253], :) ./ AR([1 : 252], :) - 1);  % daily returns
ER = sum(RR);                         % expected returns
SigR = sqrt(252) * std(RR, 1);        % stdevs of return
points_plotted = sum(ER >= 13.5 & ER <= 14.1 & SigR >= 2.6 & SigR <= 2.9);

%% special (e, sig) points
pF154 = P(:, 1);
aF154 = A * pF154;
rF154 = 100 * (aF154(2 : 253) ./ aF154(1 : 252) - 1);
eF154 = sum(rF154);                   % eF154   = 14.0185 = eXX below
sigF154 = sqrt(252) * std(rF154, 1);  % sigF154 =  2.7737
pX = P(:, 2);
aX = A * pX;
rX = 100 * (aX(2 : 253) ./ aX(1 : 252) - 1);
eX = sum(rX);                         % eX      = 13.5429
sigX = sqrt(252) * std(rX, 1);        % sigX    =  2.6496
eXX = XX(1);                          % eXX     = 14.0185
sigXX = XX(2);                        % sigXX    = 2.7052

% obtainable/unobtainable patch
EFP = [EF, 14.1 , 13.5];
SigFP = [SigF, 2.9, 2.9];

% obtainable patch
ETP = [ET, 14.1 , 13.5];
SigTP = [SigT, 2.9, 2.9];

% parameter text
points = sprintf("%-9s =%6d", "plotted", points_plotted);
nRtxt = sprintf("%-10s =%6d", "nR", nR);
deltatxt = sprintf("%-10s = %6.3f", "delta", delta);

close all;
hold on;

patch(EFP, SigFP, [0.925, 0.886, 0.843], "edgecolor", "none");
patch(ETP, SigTP, [0.85, 0.85, 1], "edgecolor", "none");

rectx = [13.5, 14.1, 14.1, 13.5, 13.5];
recty = [2.6, 2.6, 2.9, 2.9, 2.6];
plot(ER, SigR, ".b", "markersize", 2, ...
  rectx, recty, "-k", ...
  EF, SigF, "-", "color", [0.420 0.325 0.220], ...
  eF154, sigF154, "o", "markersize", 4, ...
    "markeredgecolor", "black", "markerfacecolor", "red", ...
  eX, sigX, "o", "markersize", 4, ...
    "markeredgecolor", "black", "markerfacecolor", "yellow", ...
  eXX, sigXX, ".k", "markersize", 10 ...
);
axis([13.5, 14.1, 2.6, 2.9], "equal");
text(eF154 - 0.016, sigF154 - 0.0155, ...
  "F154", "fontsize", 12, "fontweight", "bold");
text(eXX - 0.01, sigXX - 0.015, ...
  "XX", "fontsize", 12, "fontweight", "bold");
text(eX - 0.0051, sigX - 0.015, ...
  "X", "fontsize", 12, "fontweight", "bold");
text(13.79, 2.636, "false efficient frontier", "fontsize", 12, ...
  "rotation", 14);
text(13.74, 2.660, "unobtainable (e, sig)", "fontsize", 12, ...
  "rotation", 14);
text(13.70, 2.677, "true efficient frontier", "fontsize", 12, ...
  "rotation", 15);
text(13.56, 2.83, "obtainable (e, sig)", "fontsize", 12, ...
  "rotation", 14);
text(13.96, 2.66, nRtxt, "fontsize", 12);
text(13.96, 2.64, points, "fontsize", 12);
text(13.96, 2.62, deltatxt, "fontsize", 12);
set(gca, "tickdir", "out", "xtick", [13.5 : 0.1 : 14.1], ...
  "ytick", [2.6, 2.7, 2.8, 2.9], ...
  "xlabel", "annualized mean daily return", ...
  "ylabel", "annualized standard deviation of daily return", ...
  "title", "New Markowitz: how not to do mean-variance");
set(get(gca, "xlabel"), "fontsize", 12);
set(get(gca, "ylabel"), "fontsize", 12);
set(get(gca, "title"), "fontsize", 14);

hold off;
